import angr

import logging

l = logging.getLogger("angr.tests.test_rol")

import os

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_rol_x86_64():
    binary_path = os.path.join(test_location, "x86_64", "test_rol.exe")

    proj = angr.Project(binary_path, auto_load_libs=False)

    initial_state = proj.factory.blank_state(addr=0x401000)
    r_rax = initial_state.solver.BVS("rax", 64)
    initial_state.regs.rax = r_rax

    pg = proj.factory.simulation_manager(initial_state)
    pg.explore(find=0x401013, avoid=0x401010)
    found_state = pg.found[0]

    result = found_state.solver.eval(r_rax)
    assert result == 0x37B7AB70


def test_rol_i386():
    binary_path = os.path.join(test_location, "i386", "test_rol.exe")

    proj = angr.Project(binary_path, auto_load_libs=False)

    initial_state = proj.factory.blank_state(addr=0x401000)
    r_eax = initial_state.solver.BVS("eax", 32)
    initial_state.regs.eax = r_eax

    pg = proj.factory.simulation_manager(initial_state)
    pg.explore(find=0x401013, avoid=0x401010)
    found_state = pg.found[0]

    result = found_state.solver.eval(r_eax)
    assert result == 0x37B7AB70


if __name__ == "__main__":
    test_rol_x86_64()
    test_rol_i386()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("api-ms-win-core-psm-appnotify-l1-1-1.dll")
prototypes = \
    {
        #
        'RegisterAppConstrainedChangeNotification': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypeChar(label="Byte"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Constrained", "Context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimStruct({}, name="_APPCONSTRAIN_REGISTRATION", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["Routine", "Context", "Registration"]),
        #
        'UnregisterAppConstrainedChangeNotification': SimTypeFunction([SimTypePointer(SimStruct({}, name="_APPCONSTRAIN_REGISTRATION", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["Registration"]),
    }

lib.set_prototypes(prototypes)



import angr

# def broken_symvalue():
#   # concrete symvalue
#   zero = SimValue(se.BVV(0, 64))
#   assert not zero.is_symbolic()
#   assert zero.eval() == 0
#
#   # symbolic symvalue
#   x = se.BVS('x', 64)
#   sym = SimValue(x, constraints = [ x > 100, x < 200 ])
#   assert sym.is_symbolic()
#   assert sym.min_int() == 101
#   assert sym.max_int() == 199
#   assert sym.eval_upto(99) == range(101, 200)


def test_concretization_strategies():
    initial_memory = {0: b"A", 1: b"B", 2: b"C", 3: b"D"}

    s = angr.SimState(arch="AMD64", dict_memory_backer=initial_memory)

    # sanity check
    assert s.solver.eval_upto(s.memory.load(3, size=1), 2, cast_to=bytes) == [b"D"]

    x = s.solver.BVS("x", s.arch.bits)
    s.add_constraints(x >= 1)
    s.add_constraints(x <= 3)

    ss = s.copy()
    assert tuple(sorted(ss.solver.eval_upto(ss.memory.load(x, 1), 10, cast_to=bytes))) == (b"B", b"C", b"D")

    ss = s.copy()
    x = s.solver.BVS("x", s.arch.bits)
    s.add_constraints(x >= 1)
    ss.options.add(angr.options.CONSERVATIVE_READ_STRATEGY)
    ss.memory._create_default_read_strategies()
    assert "symbolic" in next(iter(ss.memory.load(x, 1).variables))


# def test_concretization():
#   s = angr.SimState(arch="AMD64", mode="symbolic")
#   dst = s.solver.BVV(0x41424300, 32)
#   dst_addr = s.solver.BVV(0x1000, 64)
#   s.memory.store(dst_addr, dst, 4)
#
#   print "MEM KEYS", s.memory.mem.keys()
#   print "REG KEYS", s.registers.mem.keys()
#
#   print "TO NATIVE..."
#   s.set_native(True)
#   print "... done"
#
#   vv = s.native_env.vexecute(pyvex.IRExpr.Load("Iend_BE", "Ity_I32", pyvex.IRExpr.Const(pyvex.IRConst.U64(0x1000))))
#   assert vv.str[:4] == 'ABC\x00'
#   s.native_env.vexecute(pyvex.IRSB(bytes='\xb8\x41\x42\x43\x44'))
#
#   #import IPython; IPython.embed()
#   print "FROM NATIVE..."
#   s.set_native(False)
#   print "... done"
#
#   assert s.reg_value(16).solver.eval() == 0x44434241
#   print "YEAH"


def broken_symbolic_write():
    s = angr.SimState(arch="AMD64", mode="symbolic")

    addr = s.solver.BVS("addr", 64)
    s.add_constraints(s.solver.Or(addr == 10, addr == 20, addr == 30))
    assert len(s.solver.eval_upto(addr, 10)) == 3

    s.memory.store(10, s.solver.BVV(1, 8))
    s.memory.store(20, s.solver.BVV(2, 8))
    s.memory.store(30, s.solver.BVV(3, 8))

    assert s.solver.unique(s.memory.load(10, 1))
    assert s.solver.unique(s.memory.load(20, 1))
    assert s.solver.unique(s.memory.load(30, 1))

    # print "CONSTRAINTS BEFORE:", s.constraints._solver.constraints
    # s.memory.store(addr, s.solver.BVV(255, 8), strategy=['symbolic','any'], limit=100)
    s.memory.store(addr, s.solver.BVV(255, 8))
    assert s.satisfiable()

    assert len(s.solver.eval_upto(addr, 10)) == 3
    assert s.solver.eval_upto(s.memory.load(10, 1), 3) == [1, 255]
    assert s.solver.eval_upto(s.memory.load(20, 1), 3) == [2, 255]
    assert s.solver.eval_upto(s.memory.load(30, 1), 3) == [3, 255]
    assert len(s.solver.eval_upto(addr, 10)) == 3

    # see if it works when constraining the write address
    sa = s.copy()
    sa.add_constraints(addr == 20)
    assert sa.satisfiable()
    assert sa.solver.eval_upto(sa.memory.load(10, 1), 3) == [1]
    assert sa.solver.eval_upto(sa.memory.load(20, 1), 3) == [255]
    assert sa.solver.eval_upto(sa.memory.load(30, 1), 3) == [3]
    assert sa.solver.eval_upto(addr, 10) == [20]

    # see if it works when constraining a value to the written one
    sv = s.copy()
    sv.add_constraints(sv.memory.load(30, 1) == 255)
    assert sv.satisfiable()
    assert sv.solver.eval_upto(sv.memory.load(10, 1), 3) == [1]
    assert sv.solver.eval_upto(sv.memory.load(20, 1), 3) == [2]
    assert sv.solver.eval_upto(sv.memory.load(30, 1), 3) == [255]
    assert sv.solver.eval_upto(addr, 10) == [30]

    # see if it works when constraining a value to the unwritten one
    sv = s.copy()
    sv.add_constraints(sv.memory.load(30, 1) == 3)
    assert sv.satisfiable()
    assert sv.solver.eval_upto(sv.memory.load(10, 1), 3) == [1, 255]
    assert sv.solver.eval_upto(sv.memory.load(20, 1), 3) == [2, 255]
    assert sv.solver.eval_upto(sv.memory.load(30, 1), 3) == [3]
    assert sv.solver.eval_upto(addr, 10) == [10, 20]

    s = angr.SimState(arch="AMD64", mode="symbolic")
    s.memory.store(0, s.solver.BVV(0x4141414141414141, 64))
    length = s.solver.BVS("length", 32)
    # s.memory.store(0, s.solver.BVV(0x4242424242424242, 64), symbolic_length=length)
    s.memory.store(0, s.solver.BVV(0x4242424242424242, 64))

    for i in range(8):
        ss = s.copy()
        ss.add_constraints(length == i)
        assert ss.solver.eval(s.memory.load(0, 8), cast_to=bytes) == b"B" * i + b"A" * (8 - i)


def test_unsat_core():
    s = angr.SimState(arch="AMD64", mode="symbolic", add_options={angr.options.CONSTRAINT_TRACKING_IN_SOLVER})
    x = s.solver.BVS("x", 32)
    s.add_constraints(s.solver.BVV(0, 32) == x)
    s.add_constraints(s.solver.BVV(1, 32) == x)

    assert not s.satisfiable()
    unsat_core = s.solver.unsat_core()
    assert len(unsat_core) == 2


if __name__ == "__main__":
    test_unsat_core()
    test_concretization_strategies()



import os

import angr
import angr.analyses.decompiler

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_smoketest():
    p = angr.Project(os.path.join(test_location, "x86_64", "all"), auto_load_libs=False)
    cfg = p.analyses.CFG(normalize=True)

    main_func = cfg.kb.functions["main"]

    _ = p.analyses.RegionIdentifier(main_func)


if __name__ == "__main__":
    test_smoketest()



import os

import archinfo

import angr
from angr.procedures.definitions.user32 import lib
from angr.sim_type import SimTypeFunction, SimTypeInt, SimTypePointer, SimTypeChar
from angr.engines.successors import SimSuccessors
from angr.calling_conventions import SimCCStdcall, SimStackArg

binaries_base = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_stub_procedure_args():
    # stub procedures should have the right number of arguments

    lib.set_prototype(
        "____a_random_stdcall_function__",
        SimTypeFunction(
            [SimTypeInt(signed=True), SimTypeInt(signed=True), SimTypeInt(signed=False)],
            SimTypePointer(SimTypeChar(), offset=0),
            arg_names=["_random_arg_0", "_random_arg_1", "_random_arg_2"],
        ),
    )
    stub = lib.get_stub("____a_random_stdcall_function__", archinfo.ArchX86())
    stub.cc = SimCCStdcall(archinfo.ArchX86())
    lib._apply_metadata(stub, archinfo.ArchX86())
    assert len(stub.prototype.args) == 3
    assert all(isinstance(arg, SimStackArg) for arg in stub.cc.arg_locs(stub.prototype))

    proj = angr.Project(os.path.join(binaries_base, "i386", "all"), auto_load_libs=False)
    state = proj.factory.blank_state()

    initial_sp = state.regs.sp
    stub.state = state
    stub.successors = SimSuccessors(0, state)
    stub.ret(0)

    succ = stub.successors.all_successors[0]
    assert state.solver.eval_one(succ.regs.sp - initial_sp) == 0x10


if __name__ == "__main__":
    test_stub_procedure_args()



import angr
import claripy

import os

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_i386():
    p = angr.Project(os.path.join(test_location, "i386", "test_strcasecmp"), auto_load_libs=False)
    arg1 = claripy.BVS("arg1", 20 * 8)
    s = p.factory.entry_state(args=("test_strcasecmp", arg1))
    sm = p.factory.simulation_manager(s)
    sm.explore()

    sm.move("deadended", "found", filter_func=lambda s: b"Welcome" in s.posix.dumps(1))

    assert len(sm.found) == 1

    f = sm.found[0]
    sol = f.solver.eval(arg1, cast_to=bytes)
    assert b"\x00" in sol
    assert sol[: sol.index(b"\x00")].lower() == b"letmein"
    assert b"wchar works" in f.posix.dumps(1)


if __name__ == "__main__":
    test_i386()



import angr
import networkx

import os

location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_kb_plugins():
    p = angr.Project(os.path.join(location, "x86_64", "fauxware"), auto_load_libs=False)

    assert isinstance(p.kb.data, angr.knowledge_plugins.Data)
    assert isinstance(p.kb.functions, angr.knowledge_plugins.FunctionManager)
    assert isinstance(p.kb.variables, angr.knowledge_plugins.VariableManager)
    assert isinstance(p.kb.labels, angr.knowledge_plugins.Labels)
    assert isinstance(p.kb.comments, angr.knowledge_plugins.Comments)

    assert isinstance(p.kb.callgraph, networkx.Graph)
    assert isinstance(p.kb.resolved_indirect_jumps, dict)
    assert isinstance(p.kb.unresolved_indirect_jumps, set)

    assert dir(p.kb) is not None
    for plugin in [
        "data",
        "functions",
        "variables",
        "labels",
        "comments",
        "callgraph",
        "resolved_indirect_jumps",
        "unresolved_indirect_jumps",
    ]:
        assert plugin in dir(p.kb)


def test_kb_plugins_typed():
    p = angr.Project(os.path.join(location, "x86_64", "fauxware"), auto_load_libs=False)

    for plugin in [
        angr.knowledge_plugins.Data,
        angr.knowledge_plugins.FunctionManager,
        angr.knowledge_plugins.VariableManager,
        angr.knowledge_plugins.Labels,
        angr.knowledge_plugins.Comments,
    ]:
        assert p.kb.get_knowledge(plugin) is None

    for plugin in [
        angr.knowledge_plugins.Data,
        angr.knowledge_plugins.FunctionManager,
        angr.knowledge_plugins.VariableManager,
        angr.knowledge_plugins.Labels,
        angr.knowledge_plugins.Comments,
    ]:
        assert isinstance(p.kb.request_knowledge(plugin), plugin)

    # The default plugins should have been instantiated by `request_knowledge`, and should now be available
    for plugin in [
        angr.knowledge_plugins.Data,
        angr.knowledge_plugins.FunctionManager,
        angr.knowledge_plugins.VariableManager,
        angr.knowledge_plugins.Labels,
        angr.knowledge_plugins.Comments,
    ]:
        assert isinstance(p.kb.request_knowledge(plugin), plugin)

    # Check that explicitly creating and registering new kind of plugin also works
    class TestPlugin(angr.knowledge_plugins.KnowledgeBasePlugin):
        def __init__(self, kb=None):
            self._kb = kb

    # Assert that unknown plugins return None when using "get_knowledge"
    assert p.kb.get_knowledge(TestPlugin) is None

    t = TestPlugin(p.kb)
    p.kb.register_plugin("test_plugin", t)

    assert p.kb.get_knowledge(TestPlugin) == t


if __name__ == "__main__":
    test_kb_plugins()



import angr
import os

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_fauxware_symbolization():
    p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
    sm = p.factory.simulation_manager()

    assert not sm.one_active.regs.rsp.symbolic

    sm.one_active.symbolizer.set_symbolization_for_all_pages()
    sm.one_active.symbolizer.resymbolize()

    # assert sm.one_active.regs.rsp.symbolic
    assert sm.one_active.symbolizer.symbolized_count

    # make sure pointers get symbolized at runtime
    n = sm.one_active.symbolizer.symbolized_count
    sm.run()
    assert not sm.errored
    assert not sm.active
    assert sm.one_deadended.symbolizer.symbolized_count > n


if __name__ == "__main__":
    test_fauxware_symbolization()
